from toga.interface import TextInput as TextInputInterface

from ..libs import *
from .base import WidgetMixin


class TextInput(TextInputInterface, WidgetMixin):
    _IMPL_CLASS = WinForms.TextBox

    def __init__(self, id=None, style=None, initial=None, placeholder=None, readonly=False, _delegate=None):
        self._delegate = _delegate
        super().__init__(id=id, style=style, initial=initial, placeholder=placeholder, readonly=readonly)
        self._create()

    def create(self):
        self._impl = self._IMPL_CLASS()
        self._impl.Multiline = False

    def _set_readonly(self, value):
        self._impl.ReadOnly = value

    def _set_placeholder(self, value):
        # self._impl.cell.placeholderString = self._placeholder
        pass

    def _get_value(self):
        return self._impl.Text

    def _set_value(self, value):
        self._impl.Text = value

    def rehint(self):
        # Height of a text input is known and fixed.
        # Width must be > 100
        # print("REHINT TextInput", self, self._impl.PreferredSize)
        self.style.hint(
            height=self._impl.PreferredSize.Height,
            min_width=100,
        )



import inspect

from .libs import *


class LongRunningTask(NSObject):
    @objc_method
    def performIteration_(self, info) -> None:
        try:
            delay = next(self.__dict__['interface'])
            NSTimer.scheduledTimerWithTimeInterval_target_selector_userInfo_repeats_(delay, self, get_selector('performIteration:'), None, False)
        except StopIteration:
            pass


def process_callback(callback_result):
    "Handle generators in actions"
    if inspect.isgenerator(callback_result):
        task = LongRunningTask.alloc().init()
        task.__dict__['interface'] = callback_result
        task.performIteration_(None)



import os

from toga.interface.app import App as AppInterface

from .libs import *

from .window import Window
# from .widgets.icon import Icon, TIBERIUS_ICON


class MainWindow(Window):
    def __init__(self, title=None, position=(100, 100), size=(640, 480)):
        super(MainWindow, self).__init__(title, position, size)


class App(AppInterface):
    _MAIN_WINDOW_CLASS = MainWindow

    def __init__(self, name, app_id, icon=None, startup=None, document_types=None):
        # Set the icon for the app
        # Icon.app_icon = Icon.load(icon, default=TIBERIUS_ICON)

        super().__init__(
            name=name,
            app_id=app_id,
            icon=None,  # Icon.app_icon,
            startup=startup,
            document_types=document_types
        )

    def _startup(self):
        self._impl = WinForms.Application
        Threading.Thread.CurrentThread.ApartmentState = Threading.ApartmentState.STA

        # self._impl.setApplicationIconImage_(self.icon._impl)

        # Set the menu for the app.
        # self._impl.setMainMenu_(self.menu)

        # Call user code to populate the main window
        self.startup()


    def open_document(self, fileURL):
        '''Add a new document to this app.'''
        print("STUB: If you want to handle opening documents, implement App.open_document(fileURL)")

    def main_loop(self):
        self._startup()
        self._impl.Run(self.main_window._impl)



from toga.interface import Button as ButtonInterface

from ..libs import *

from .base import WidgetMixin
# from ..utils import process_callback


class TogaButton(WinForms.Button):
    def __init__(self, interface):
        super().__init__()
        self.interface = interface
        self.Click += self.on_click

    def on_click(self, sender, event):
        self.interface.on_press(self.interface)


class Button(ButtonInterface, WidgetMixin):
    def __init__(self, label, id=None, style=None, on_press=None, enabled=True):
        super().__init__(label, id=id, style=style, on_press=on_press, enabled=enabled)
        self._create()

    def create(self):
        self._impl = TogaButton(self)

    def _set_label(self, label):
        self._impl.Text = self.label
        self.rehint()

    def _set_enabled(self, value):
        self._impl.Enabled = value

    def rehint(self):
        # self._impl.Size = Size(0, 0)
        # print("REHINT Button", self, self._impl.PreferredSize)
        self.style.hint(
            height=self._impl.PreferredSize.Height,
            min_width=self._impl.PreferredSize.Width,
        )



from toga.interface import Label as LabelInterface

from ..libs import *
from .base import WidgetMixin
from toga.constants import LEFT_ALIGNED


class Label(LabelInterface, WidgetMixin):
    def __init__(self, text, id=None, style=None, alignment=LEFT_ALIGNED):
        super().__init__(id=id, style=style, text=text, alignment=alignment)
        self._create()

    def create(self):
        self._impl = WinForms.Label()

    def _set_alignment(self, value):
        # self._impl.setAlignment_(NSTextAlignment(value))
        pass

    def _set_text(self, value):
        self._impl.Text = self._text

    def rehint(self):
        # Width & height of a label is known and fixed.
        # self._impl.Size = Size(0, 0)
        # print("REHINT label", self, self._impl.PreferredSize)
        self.style.hint(
            height=self._impl.PreferredSize.Height,
            width=self._impl.PreferredSize.Width,
        )



from toga.interface import Box as BoxInterface

from ..libs import *

from .base import WidgetMixin


class Box(BoxInterface, WidgetMixin):
    def __init__(self, id=id, style=None, children=None):
        super().__init__(id=id, style=style, children=children)
        self._create()

    def create(self):
        pass


from toga.interface import WebView as WebViewInterface

from .base import WidgetMixin
from ..libs import *


class TogaWebBrowser(WinForms.WebBrowser):
    def __init__(self, interface):
        super().__init__()
        self.interface = interface


class WebView(WebViewInterface, WidgetMixin):
    def __init__(self, id=None, style=None, url=None, on_key_down=None):
        super().__init__(id=id, style=style, url=url, on_key_down=on_key_down)
        self._create()

    def create(self):
        self._impl = TogaWebBrowser(self)

    def _set_url(self, value):
        if value:
            self._impl.Navigate(Uri(value))

    def _set_content(self, root_url, content):
        # self._impl.mainFrame.loadHTMLString_baseURL_(content, NSURL.URLWithString_(root_url))
        self._impl.Navigate(Uri(root_url))

    def evaluate(self, javascript):
        # return self._impl.stringByEvaluatingJavaScriptFromString_(javascript)
        pass



from toga.interface.window import Window as WindowInterface

from .libs import *

from .container import Container
from . import dialogs


class Window(WindowInterface):
    # _IMPL_CLASS = WinForms.Form
    _CONTAINER_CLASS = Container
    _DIALOG_MODULE = dialogs

    def __init__(self, title=None, position=(100, 100), size=(640, 480), toolbar=None, resizeable=True, closeable=True, minimizable=True):
        super().__init__(title=title, position=position, size=size, toolbar=toolbar, resizeable=resizeable, closeable=closeable, minimizable=minimizable)
        self._create()

    def create(self):
        self._impl = WinForms.Form(self)
        self._impl.ClientSize = Size(self._size[0], self._size[1])
        self._impl.Resize += self._on_resize

    # def _set_toolbar(self, items):

    def _set_content(self, widget):
        self._impl.Controls.Add(widget._container._impl)

    def _set_title(self, title):
        self._impl.Text = title

    def show(self):
        # The first render of the content will establish the
        # minimum possible content size; use that to enforce
        # a minimum window size.
        TITLEBAR_HEIGHT = 36
        self._impl.MinimumSize = Size(
            int(self.content.layout.width),
            int(self.content.layout.height) + TITLEBAR_HEIGHT
        )

        # Set the size of the container to be the same as the window
        self._container._impl.Size = self._impl.ClientSize

        # Do the first layout render.
        self._container._update_layout(
            width=self._impl.ClientSize.Width,
            height=self._impl.ClientSize.Height,
        )

    def close(self):
        self._impl.Close()


    def _on_resize(self, sender, args):
        if self.content:
            # Set the size of the container to be the same as the window
            self._container._impl.Size = self._impl.ClientSize
            # Re-layout the content
            self.content._update_layout(
                width=sender.ClientSize.Width,
                height=sender.ClientSize.Height,
            )



# from ..container import Constraints
from ..libs import *


class WidgetMixin:
    def _set_app(self, app):
        pass

    def _set_window(self, window):
        pass

    def _set_container(self, container):
        if self._impl:
            self._container._impl.Controls.Add(self._impl)

        self.rehint()

    def _add_child(self, child):
        if self._container:
            child._set_container(self._container)

    def _apply_layout(self):
        if self._impl:
            self._impl.Size = Size(int(self.layout.width), int(self.layout.height))
            self._impl.Location = Point(int(self.layout.absolute.left), int(self.layout.absolute.top))

    def rehint(self):
        pass

    def _set_font(self, font):
        pass
